require 'ffi'
require 'rbconfig'

module IupLib # :nodoc:
  extend FFI::Library

  this_dir = File.dirname(__FILE__)
  case RbConfig::CONFIG['host_os']
  when /mswin|mingw/
    ffi_lib 'lib/library/iup.dll'
  when /linux|cygwin/
    ffi_lib 'libiup'
  else
    raise Exception, 'iup libraries not found for this platform'
  end

  # Attributes
  attach_function :IupSetAttribute, [:pointer, :string, :string], :void
  attach_function :IupSetAttributes, [:pointer, :string], :pointer
  attach_function :IupGetAttribute, [:pointer, :string], :strptr
  attach_function :IupSetAttributeHandle, [:pointer, :string, :pointer], :void
  attach_function :IupSetGlobal, [:string, :string], :void
  attach_function :IupGetGlobal, [:string], :strptr

  callback :iup_callback, [], :int
  attach_function :IupSetFunction, [:string, :iup_callback], :iup_callback

  attach_function :IupGetInt, [:pointer, :string], :int

  # Events
  attach_function :IupOpen, [:int, :pointer], :int
  attach_function :IupMainLoop, [], :void
  attach_function :IupClose, [], :void
  attach_function :IupDestroy, [:pointer], :void
  attach_function :IupRedraw, [:pointer], :void

  attach_function :IupMap, [:pointer], :void

  # -- Callbacks
  callback :iup_callback_d_i, [:pointer, :double], :int
  callback :iup_callback_ff_i, [:pointer, :float, :float], :int
  callback :iup_callback_fiis_i, [:pointer, :float, :int, :int, :string], :int
  callback :iup_callback_s_i, [:pointer, :string], :int
  callback :iup_callback_sii_i, [:pointer, :string, :int, :int], :int
  callback :iup_callback_i_i, [:pointer, :int], :int
  callback :iup_callback_i_s, [:pointer, :int], :strptr
  callback :iup_callback_iff_i, [:pointer, :int, :float, :float], :int
  callback :iup_callback_ii_i, [:pointer, :int, :int], :int
  callback :iup_callback_iiff_i, [:pointer, :int, :int, :float, :float], :int
  callback :iup_callback_iiffi_i, [:pointer, :int, :int, :float, :float, :int], :int
  callback :iup_callback_iiffff_i, [:pointer, :int, :int, :float, :float, :float, :float], :int
  callback :iup_callback_is_i, [:pointer, :int, :string], :int
  callback :iup_callback_iis_i, [:pointer, :int, :int, :string], :int
  callback :iup_callback_iii_i, [:pointer, :int, :int, :int], :int
  callback :iup_callback_iiii_i, [:pointer, :int, :int, :int, :int], :int
  callback :iup_callback_iiis_i, [:pointer, :int, :int, :string], :int
  callback :iup_callback_iis_i, [:pointer, :int, :int, :string], :int
  callback :iup_callback_iiiis_i, [:pointer, :int, :int, :int, :int, :string], :int
  callback :iup_callback_plain, [:pointer], :int
  callback :iup_callback_plain_v, [:pointer], :void
  callback :iup_callback_p_i, [:pointer, :pointer], :int
  callback :iup_callback_pp_i, [:pointer, :pointer, :pointer], :int
  callback :iup_callback_ss_i, [:pointer, :string, :string], :int
  callback :iup_callback_ssi_i, [:pointer, :string, :string, :int], :int
  callback :iup_callback_ssiii_i, [:pointer, :string, :string, :int, :int, :int], :int

  attach_function :IupSetCallbackD_I, :IupSetCallback, [:pointer, :string, :iup_callback_d_i], :iup_callback_d_i
  attach_function :IupSetCallbackFF_I, :IupSetCallback, [:pointer, :string, :iup_callback_ff_i], :iup_callback_ff_i
  attach_function :IupSetCallbackFIIS_I, :IupSetCallback, [:pointer, :string, :iup_callback_fiis_i], :iup_callback_fiis_i
  attach_function :IupSetCallbackS_I, :IupSetCallback, [:pointer, :string, :iup_callback_s_i], :iup_callback_s_i
  attach_function :IupSetCallbackSII_I, :IupSetCallback, [:pointer, :string, :iup_callback_sii_i], :iup_callback_sii_i
  attach_function :IupSetCallbackI_I, :IupSetCallback, [:pointer, :string, :iup_callback_i_i], :iup_callback_i_i
  attach_function :IupSetCallbackI_S, :IupSetCallback, [:pointer, :string, :iup_callback_i_s], :iup_callback_i_s
  attach_function :IupSetCallbackIFF_I, :IupSetCallback, [:pointer, :string, :iup_callback_iff_i], :iup_callback_iff_i
  attach_function :IupSetCallbackII_I, :IupSetCallback, [:pointer, :string, :iup_callback_ii_i], :iup_callback_ii_i
  attach_function :IupSetCallbackIIFF_I, :IupSetCallback, [:pointer, :string, :iup_callback_iiff_i], :iup_callback_iiff_i
  attach_function :IupSetCallbackIIFFI_I, :IupSetCallback, [:pointer, :string, :iup_callback_iiffi_i], :iup_callback_iiffi_i
  attach_function :IupSetCallbackIIFFFF_I, :IupSetCallback, [:pointer, :string, :iup_callback_iiffff_i], :iup_callback_iiffff_i
  attach_function :IupSetCallbackIS_I, :IupSetCallback, [:pointer, :string, :iup_callback_is_i], :iup_callback_is_i
  attach_function :IupSetCallbackIIS_I, :IupSetCallback, [:pointer, :string, :iup_callback_iis_i], :iup_callback_iis_i
  attach_function :IupSetCallbackIII_I, :IupSetCallback, [:pointer, :string, :iup_callback_iii_i], :iup_callback_iii_i
  attach_function :IupSetCallbackIIII_I, :IupSetCallback, [:pointer, :string, :iup_callback_iiii_i], :iup_callback_iiii_i
  attach_function :IupSetCallbackIIS_I, :IupSetCallback, [:pointer, :string, :iup_callback_iis_i], :iup_callback_iis_i
  attach_function :IupSetCallbackIIIIS_I, :IupSetCallback, [:pointer, :string, :iup_callback_iiiis_i], :iup_callback_iiiis_i
  attach_function :IupSetCallbackPlain, :IupSetCallback, [:pointer, :string, :iup_callback_plain], :iup_callback_plain
  attach_function :IupSetCallbackPlain_V, :IupSetCallback, [:pointer, :string, :iup_callback_plain_v], :iup_callback_plain_v
  attach_function :IupSetCallbackP_I, :IupSetCallback, [:pointer, :string, :iup_callback_p_i], :iup_callback_p_i
  attach_function :IupSetCallbackPP_I, :IupSetCallback, [:pointer, :string, :iup_callback_pp_i], :iup_callback_pp_i
  attach_function :IupSetCallbackSS_I, :IupSetCallback, [:pointer, :string, :iup_callback_ss_i], :iup_callback_ss_i
  attach_function :IupSetCallbackSSI_I, :IupSetCallback, [:pointer, :string, :iup_callback_ssi_i], :iup_callback_ssi_i
  attach_function :IupSetCallbackSSIII_I, :IupSetCallback, [:pointer, :string, :iup_callback_ssiii_i], :iup_callback_ssiii_i

  # Layout 
  # -- Composition
  attach_function :IupFill, [], :pointer
  attach_function :IupGridBox, [:pointer, :varargs], :pointer
  attach_function :IupVbox, [:pointer, :varargs], :pointer
  attach_function :IupHbox, [:pointer, :varargs], :pointer
  attach_function :IupZbox, [:pointer, :varargs], :pointer
  attach_function :IupRadio, [:pointer], :pointer
  attach_function :IupNormalizer, [:pointer, :varargs], :pointer
  attach_function :IupBackgroundBox, [:pointer], :pointer
  attach_function :IupCbox, [:pointer, :varargs], :pointer
  attach_function :IupDetachBox, [:pointer], :pointer
  attach_function :IupExpander, [:pointer], :pointer
  attach_function :IupSbox, [:pointer], :pointer
  attach_function :IupScrollBox, [:pointer], :pointer
  attach_function :IupSplit, [:pointer, :pointer], :pointer
  # -- Hierarchy
  attach_function :IupAppend, [:pointer, :pointer], :pointer
  attach_function :IupDetach, [:pointer], :void
  attach_function :IupInsert, [:pointer, :pointer, :pointer], :pointer
  attach_function :IupReparent, [:pointer, :pointer, :pointer], :int
  attach_function :IupGetParent, [:pointer], :pointer
  attach_function :IupGetChild, [:pointer, :int], :pointer
  attach_function :IupGetChildPos, [:pointer, :pointer], :int
  attach_function :IupGetChildCount, [:pointer], :int
  attach_function :IupGetNextChild, [:pointer, :pointer], :pointer
  attach_function :IupGetBrother, [:pointer], :pointer
  attach_function :IupGetDialog, [:pointer], :pointer
  attach_function :IupGetDialogChild, [:pointer, :string], :pointer
  # -- Utilities
  attach_function :IupRefresh, [:pointer], :void
  attach_function :IupRefreshChildren, [:pointer], :void
  attach_function :IupUpdate, [:pointer], :void
  attach_function :IupRedraw, [:pointer, :int], :void
  attach_function :IupConvertXYToPos, [:pointer, :int, :int], :int

  # Dialogs
  # -- reference
  attach_function :IupDialog, [:pointer], :pointer
  attach_function :IupPopup, [:pointer, :int, :int], :int
  attach_function :IupShow, [:pointer], :void
  attach_function :IupShowXY, [:pointer, :int, :int], :int
  attach_function :IupHide, [:pointer], :void

  # -- predefined
  attach_function :IupFileDlg, [], :pointer
  attach_function :IupMessageDlg, [], :pointer
  attach_function :IupColorDlg, [], :pointer
  attach_function :IupFontDlg, [], :pointer
  attach_function :IupProgressDlg, [], :pointer
  attach_function :IupAlarm, [:string, :string, :string, :string, :string], :int
  attach_function :IupGetFile, [:string], :int
  attach_function :IupGetColor, [:int, :int, :pointer, :pointer, :pointer], :int
  attach_function :IupGetText, [:pointer, :pointer], :int
  attach_function :IupListDialog, [:int, :string, :int, :pointer, :int, :int, :int, :pointer], :int
  attach_function :IupMessage, [:string, :string], :void

  # Controls
  # -- standard
  attach_function :IupButton, [:string, :string], :pointer
  attach_function :IupCanvas, [:string], :pointer
  attach_function :IupFrame, [:pointer], :pointer
  attach_function :IupLabel, [:string], :pointer
  attach_function :IupLink, [:string, :string], :pointer
  attach_function :IupList, [:string], :pointer
  attach_function :IupProgressBar, [], :pointer
  attach_function :IupSpin, [], :pointer
  attach_function :IupTabs, [:pointer, :varargs], :pointer
  attach_function :IupText, [:string], :pointer
  attach_function :IupToggle, [:string, :string], :pointer
  attach_function :IupTree, [], :pointer
  attach_function :IupVal, [:string], :pointer

  # Resources
  # -- images
  attach_function :IupImage, [:int, :int, :pointer], :pointer
  attach_function :IupImageRGB, [:int, :int, :pointer], :pointer
  attach_function :IupImageRGBA, [:int, :int, :pointer], :pointer
  attach_function :IupSaveImageAsText, [:pointer, :string, :string, :string], :int
  # -- keyboard
  attach_function :IupNextField, [:pointer], :pointer
  attach_function :IupPreviousField, [:pointer], :pointer
  attach_function :IupGetFocus, [], :pointer
  attach_function :IupSetFocus, [:pointer], :pointer
  # -- menus
  attach_function :IupItem, [:string, :string], :pointer
  attach_function :IupMenu, [:pointer, :varargs], :pointer
  attach_function :IupSeparator, [], :pointer
  attach_function :IupSubmenu, [:string, :pointer], :pointer
  # -- handle names
  attach_function :IupSetHandle, [:string, :pointer], :void
  attach_function :IupGetHandle, [:string], :pointer
  attach_function :IupGetName, [:pointer], :strptr
  # -- string names
  attach_function :IupSetLanguage, [:string], :void
  attach_function :IupGetLanguage, [], :strptr

  attach_function :IupClipboard, [], :pointer
  attach_function :IupTimer, [], :pointer
  attach_function :IupHelp, [:string], :int

  # helper functions, to create memory structures from arrays

  # Input:: an array of chars
  # Output:: pointer to memory holding array of chars
  def IupLib.pointer_from_chars array
    ptr = FFI::MemoryPointer.new :char, array.size
    ptr.write_array_of_char array
    return ptr
  end

  # Input:: an array of ints
  # Output:: pointer to memory holding array of ints
  def IupLib.pointer_from_ints array
    ptr = FFI::MemoryPointer.new :int, array.size
    ptr.write_array_of_int array
    return ptr
  end
end

