module Iup

  # Used to contain a child widget, but itself provides no visible decorations.
  #
  # == Attributes
  #
  # border:: Shows a border around the box, values 'yes' / 'no'.
  # canfocus:: Value is read-only and always 'no'.
  # clientoffset:: read-only, returns current offset of box in its client
  #                as "widthxheight".
  # clientsize:: read-only, returns current size of box as "widthxheight".
  # expand:: Allows box to fill available space in indicated direction.
  #          Values 'no' / 'horizontal' / 'vertical' / 'yes'.
  #
  class BackgroundBox < Widget

    # Creates an instance of the box.
    # widget:: the child widget to contain
    # block:: optional block to set up the box's attributes.
    def initialize widget, &block
      @handle = IupLib.IupBackgroundBox widget.handle

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end

    define_attribute :border
    define_readonly :canfocus
    define_attribute :clientoffset
    define_attribute :clientsize
    define_attribute :expand
  end
end
