module Iup

  # A ColourDialog allows the user to select a colour.
  #
  # Also see: Iup#get_colour
  #
  # == Attributes
  #
  # alpha:: If defined, enables alpha selection in dialog. 
  #         Returns value if user pressed 'OK'.
  # colortable:: "m;n;..." gives a list of values for the palette.
  # parentdialog:: This dialog will be always in front of the parent dialog. 
  #                If the parent is minimized, this dialog is automatically minimized.
  #                *Important* Closing the parent will also close the child, but the 
  #                child dialog's CLOSE_CB method will not be called.
  # showalpha:: If set, shows the colour table. Values 'yes' / 'no'.
  # showcolortable:: If set, shows the colour table. Values 'yes' / 'no'.
  # showhelp:: Shows a help button if +help_cb+ defined. Values 'yes' / 'no'.
  # showhex:: If set, shows a hexadecimal representation of colour. Values 'yes' / 'no'.
  # status:: <b>read-only</b> Returns '1' if 'OK' pressed, or null.
  # title:: Title text for the dialog.
  # value:: Initial value for dialog, and return value if 'OK' pressed, 
  #         as 'r g b' or 'r g b a'.
  # valuehex:: Initial value for dialog, and return value if 'OK' pressed, 
  #            as 'rrggbb'.
  # valuehsi:: Initial value for dialog, and return value if 'OK' pressed, 
  #            as 'H S I'.
  #
  class ColourDialog < Widget

    # Creates a dialog, using the optional block to set its attributes.
    def initialize &block
      @handle = IupLib.IupColorDlg

      self.instance_eval &block if block_given?
    end

    # Shows the dialog at position x, y.
    def popup x=0, y=0
      IupLib.IupPopup @handle, x, y
    end

    # -- attributes

    define_attribute :alpha
    define_attribute :colortable
    
    def parentdialog parent=nil # :nodoc:
      attribute_reference 'PARENTDIALOG', Dialog, parent
    end

    define_attribute :showalpha
    define_attribute :showcolortable
    define_attribute :showhelp
    define_attribute :showhex
    define_readonly :status
    define_attribute :title
    define_attribute :value
    define_attribute :valuehex
    define_attribute :valuehsi
  end
end

