module Iup

  # Defines attributes common to most widgets
  #
  # == Attributes
  #
  # active:: Enables or prevents user interaction.
  #          Typically, a widget will appear greyed-out when inactive.
  #          Values 'yes' / 'no'.
  # bgcolor:: Specify RGB values for background colour, as "r g b".
  # fgcolor:: Specify RGB values for foreground colour, as "r g b".
  # font:: Sets the widget's font, using a font description.
  # maxsize:: Maximum size of widget, value as "widthxheight".
  # minsize:: Minimum size of widget, value as "widthxheight".
  # size:: Size of the widget, in character units, value as "widthxheight".
  # visible:: Shows or hides the widget. Values 'yes' / 'no'.
  # wid:: <b>read-only</b> Native widget identifier.
  # zorder:: <b>write-only</b> Alters z-order, values as 'top' / 'bottom'.
  #
  module CommonAttributes
    extend AttributeBuilders
    
    define_attribute :active
    define_attribute :bgcolor
    define_attribute :fgcolor
    define_attribute :font
    define_attribute :visible

    define_attribute :maxsize
    define_attribute :minsize
    define_attribute :size

    define_readonly :wid
    define_writeonly :zorder

  end

  # Standard callback method for when a mouse button is pressed or released.
  module ButtonCallback

    # Action generated when any mouse button is pressed or released.
    # button_cb takes a callback which accepts 5 arguments (button, state, x, y, status)
    # button:: the number of the button that changed, one of: BUTTON1 (left), BUTTON2 (middle), BUTTON3 (right)
    # state:: 1 for 'pressed', 0 for 'released'
    # x:: x-coordinate 
    # y:: y-coordinate
    # status:: status of the mouse buttons and some keyboard keys at the moment the event is generated.
    #-- 
    # TODO: include status test methods iup_iscontrol etc
    #
    def button_cb callback
      unless callback.arity == 5
        raise ArgumentError, 'button_cb callback must take 5 arguments: (button, state, x, y, status)'
      end
      cb = Proc.new do |ih, b, p, x, y, status|
        callback.call b.chr, p, x, y, status
      end
      define_callback cb, 'BUTTON_CB', :iiiis_i
    end

  end

end

