# Iup is the main module, in which all controls and widgets are named.
#
# The module also provides constants and supporting methods.
#
# == Constants
#--
# TODO: Check if constants are used in single or multiple places.
# If single (e.g. canvas / scrollbars), they can be moved to those files.
#++
#
# === predefined colours
#
# * CD_RED
# * CD_DARK_RED
# * CD_GREEN
# * CD_DARK_GREEN
# * CD_BLUE
# * CD_DARK_BLUE
#
# * CD_YELLOW
# * CD_DARK_YELLOW 
# * CD_MAGENTA 
# * CD_DARK_MAGENTA
# * CD_CYAN 
# * CD_DARK_CYAN
#
# * CD_WHITE 
# * CD_BLACK
#
# * CD_DARK_GRAY
# * CD_GRAY
#
#
# === operations on vertical scrollbar (ScrollBarAttributes#scroll_cb)
#
# * IUP_SBUP : line up
# * IUP_SBDN : line down
# * IUP_SBPGUP : page up
# * IUP_SBPGDN : page down
# * IUP_SBPOSV : vertical positioning
# * IUP_SBDRAGV : vertical drag
#
# === operations on horizontal scrollbar (ScrollBarAttributes#scroll_cb)
#
# * IUP_SBLEFT : column left
# * IUP_SBRIGHT : column right
# * IUP_SBPGLEFT : page left
# * IUP_SBPGRIGHT : page right
# * IUP_SBPOSH : horizontal positioning
# * IUP_SBDRAGH : horizontal drag
#
#
# === line style 
#
# * CD_CONTINUOUS
# * CD_DASHED
# * CD_DOTTED
# * CD_DASH_DOT 
# * CD_DASH_DOT_DOT 
# * CD_CUSTOM 
#
# === line join
#
# * CD_MITER
# * CD_BEVEL
# * CD_ROUND
#
# === line cap 
#
# * CD_CAPFLAT
# * CD_CAPSQUARE
# * CD_CAPROUND
#
# === mark type 
#
# * CD_PLUS
# * CD_STAR
# * CD_CIRCLE
# * CD_X 
# * CD_BOX 
# * CD_DIAMOND
# * CD_HOLLOW_CIRCLE
# * CD_HOLLOW_BOX
# * CD_HOLLOW_DIAMOND
#
# === background opacity mode 
#
# * CD_OPAQUE
# * CD_TRANSPARENT
#
# === fill mode
#
# * CD_EVENODD
# * CD_WINDING
#
# === hatch type
#
# * CD_HORIZONTAL 
# * CD_VERTICAL 
# * CD_FDIAGONAL
# * CD_BDIAGONAL
# * CD_CROSS
# * CD_DIAGCROSS
#
# === interior style
# 
# * CD_SOLID
# * CD_HATCH
# * CD_STIPPLE
# * CD_PATTERN
# * CD_HOLLOW
# 
# === path actions
#
# * CD_PATH_NEW
# * CD_PATH_MOVETO
# * CD_PATH_LINETO
# * CD_PATH_ARC
# * CD_PATH_CURVETO
# * CD_PATH_CLOSE
# * CD_PATH_FILL
# * CD_PATH_STROKE
# * CD_PATH_FILLSTROKE
# * CD_PATH_CLIP
#
# === polygon mode (begin...end)
#
# * CD_FILL
# * CD_OPEN_LINES
# * CD_CLOSED_LINES
# * CD_CLIP
# * CD_BEZIER
# * CD_REGION
# * CD_PATH
#
# * CD_POLYCUSTOM
#  
# === write mode 
# 
# * CD_REPLACE
# * CD_XOR
# * CD_NOT_XOR
# 
# === text alignment 
# 
# * CD_NORTH 
# * CD_SOUTH
# * CD_EAST
# * CD_WEST
# * CD_NORTH_EAST 
# * CD_NORTH_WEST
# * CD_SOUTH_EAST
# * CD_SOUTH_WEST
# * CD_CENTER 
# * CD_BASE_LEFT
# * CD_BASE_CENTER 
# * CD_BASE_RIGHT 
#
# === text styles
#
# * CD_PLAIN : plain text
# * CD_BOLD : bold text
# * CD_ITALIC : italic text
# * CD_UNDERLINE : underline text (only windows and pdf)
# * CD_STRIKEOUT : strikeout text (only windows and pdf)
#
# * CD_BOLD_ITALIC : bold and italic text
#  
module Iup

  # Common Flags and Return Values

  ERROR = 1
  NOERROR = 0
  OPENED = -1
  INVALID = -1
  INVALID_ID = -10

  # Callback Return Values

  IGNORE = -1
  DEFAULT = -2
  CLOSE = -3
  CONTINUE = -4

  # IupPopup and IupShowXY Parameter Values

  CENTER = 0xFFFF # 65535 
  LEFT = 0xFFFE # 65534 
  RIGHT = 0xFFFD # 65533 
  MOUSEPOS = 0xFFFC # 65532 
  CURRENT = 0xFFFB # 65531 
  CENTERPARENT = 0xFFFA # 65530 
  TOP = LEFT
  BOTTOM = RIGHT

  # Mouse Button Values and Macros

  BUTTON1 = '1'
  BUTTON2 = '2'
  BUTTON3 = '3'
  BUTTON4 = '4'
  BUTTON5 = '5'

  # Pre-Defined Masks

  MASK_FLOAT = "[+/-]?(/d+/.?/d*|/./d+)"
  MASK_UFLOAT = "(/d+/.?/d*|/./d+)"
  MASK_EFLOAT = "[+/-]?(/d+/.?/d*|/./d+)([eE][+/-]?/d+)?"
  MASK_INT = "[+/-]?/d+"
  MASK_UINT = "/d+"

  # IupGetParam Callback situations

  GETPARAM_OK = -1
  GETPARAM_INIT = -2
  GETPARAM_CANCEL = -3
  GETPARAM_HELP = -4

  # Key definitions

  # from 32 to 126, all character sets are equal,
  # the key code is the same as the ASCii character code.

  K_SP =         ' '.ord
  K_exclam  =    '!'.ord
  K_quotedbl =   "\"".ord
  K_numbersign = '#'.ord
  K_dollar =     '$'.ord
  K_percent =    '%'.ord
  K_ampersand =  '&'.ord
  K_apostrophe = "\'".ord
  K_parentleft = '('.ord
  K_parentright =')'.ord
  K_asterisk =   '*'.ord
  K_plus =       '+'.ord
  K_comma =      ','.ord
  K_minus =      '-'.ord
  K_period =     '.'.ord
  K_slash =      '/'.ord
  K_0 =          '0'.ord
  K_1 =          '1'.ord
  K_2 =          '2'.ord
  K_3 =          '3'.ord
  K_4 =          '4'.ord
  K_5 =          '5'.ord
  K_6 =          '6'.ord
  K_7 =          '7'.ord
  K_8 =          '8'.ord
  K_9 =          '9'.ord
  K_colon =      ':'.ord
  K_semicolon =  ';'.ord
  K_less =       '<'.ord
  K_equal =      '='.ord
  K_greater =    '>'.ord
  K_question =   '?'.ord
  K_at =         '@'.ord
  K_A =          'A'.ord
  K_B =          'B'.ord
  K_C =          'C'.ord
  K_D =          'D'.ord
  K_E =          'E'.ord
  K_F =          'F'.ord
  K_G =          'G'.ord
  K_H =          'H'.ord
  K_I =          'I'.ord
  K_J =          'J'.ord
  K_K =          'K'.ord
  K_L =          'L'.ord
  K_M =          'M'.ord
  K_N =          'N'.ord
  K_O =          'O'.ord
  K_P =          'P'.ord
  K_Q =          'Q'.ord
  K_R =          'R'.ord
  K_S =          'S'.ord
  K_T =          'T'.ord
  K_U =          'U'.ord
  K_V =          'V'.ord
  K_W =          'W'.ord
  K_X =          'X'.ord
  K_Y =          'Y'.ord
  K_Z =          'Z'.ord
  K_bracketleft ='['.ord
  K_backslash =  "\\".ord
  K_bracketright =']'.ord
  K_circum =     '^'.ord
  K_underscore = '_'.ord
  K_grave =      '`'.ord
  K_a =          'a'.ord
  K_b =          'b'.ord
  K_c =          'c'.ord
  K_d =          'd'.ord
  K_e =          'e'.ord
  K_f =          'f'.ord
  K_g =          'g'.ord
  K_h =          'h'.ord
  K_i =          'i'.ord
  K_j =          'j'.ord
  K_k =          'k'.ord
  K_l =          'l'.ord
  K_m =          'm'.ord
  K_n =          'n'.ord
  K_o =          'o'.ord
  K_p =          'p'.ord
  K_q =          'q'.ord
  K_r =          'r'.ord
  K_s =          's'.ord
  K_t =          't'.ord
  K_u =          'u'.ord
  K_v =          'v'.ord
  K_w =          'w'.ord
  K_x =          'x'.ord
  K_y =          'y'.ord
  K_z =          'z'.ord
  K_braceleft =  '{'.ord
  K_bar =        '|'.ord
  K_braceright = '}'.ord
  K_tilde =      '~'.ord

  ## Printable ASCii keys */

  #define iup_isprint(_c) ((_c) > 31 && (_c) < 127)

  # also define the escape sequences that have keys associated 

  K_BS =    "\b".ord
  K_TAB =   "\t".ord
  K_LF =    "\n".ord
  K_CR =    "\r".ord

  # IUP Extended Key Codes, range start at 128

  #define iup_isXkey(_c)      ((_c) >= 128)

  # These use the same definition as X11 and GDK.
  #   This also means that any X11 or GDK definition can also be used. 

  K_PAUSE =    0xFF13
  K_ESC =      0xFF1B
  K_HOME =     0xFF50           
  K_LEFT =     0xFF51
  K_UP =       0xFF52
  K_RIGHT =    0xFF53
  K_DOWN =     0xFF54 
  K_PGUP =     0xFF55
  K_PGDN =     0xFF56
  K_END =      0xFF57
  K_MIDDLE =   0xFF0B
  K_Print =    0xFF61 
  K_INS =      0xFF63
  K_Menu =     0xFF67
  K_DEL =      0xFFFF
  K_F1 =       0xFFBE
  K_F2 =       0xFFBF
  K_F3 =       0xFFC0
  K_F4 =       0xFFC1
  K_F5 =       0xFFC2
  K_F6 =       0xFFC3
  K_F7 =       0xFFC4
  K_F8 =       0xFFC5
  K_F9 =       0xFFC6
  K_F10 =      0xFFC7
  K_F11 =      0xFFC8
  K_F12 =      0xFFC9

  # no Shift/Ctrl/Alt */
  K_LSHIFT =   0xFFE1
  K_RSHIFT =   0xFFE2
  K_LCTRL =    0xFFE3
  K_RCTRL =    0xFFE4
  K_LALT =     0xFFE9
  K_RALT =     0xFFEA

  K_NUM =      0xFF7F
  K_SCROLL =   0xFF14
  K_CAPS =     0xFFE5

  # Also, these are the same as the Latin-1 definition */

  K_ccedilla =  0x00E7
  K_Ccedilla =  0x00C7
  K_acute =     0x00B4  # no Shift/Ctrl/Alt */
  K_diaeresis = 0x00A8

  #*****************************************************/
  # Modifiers use last 4 bits. Since IUP 3.9           */
  # These modifiers definitions are specific to IUP    */
  #*****************************************************/

  #define iup_isShiftXkey(_c) ((_c) & 0x10000000)
  #define iup_isCtrlXkey(_c)  ((_c) & 0x20000000)
  #define iup_isAltXkey(_c)   ((_c) & 0x40000000)
  #define iup_isSysXkey(_c)   ((_c) & 0x80000000)

  #define iup_XkeyBase(_c)  ((_c) & 0x0FFFFFFF)
  #define iup_XkeyShift(_c) ((_c) | 0x10000000)   # Shift  */
  #define iup_XkeyCtrl(_c)  ((_c) | 0x20000000)   # Ctrl   */
  #define iup_XkeyAlt(_c)   ((_c) | 0x40000000)   # Alt    */
  #define iup_XkeySys(_c)   ((_c) | 0x80000000)   # Sys (Win or Apple) */

  # These definitions are here for backward compatibility 
  #   and to simplify some key combination usage.
  #   But since IUP 3.9, modifiers can be combined with any key
  #   and they can be mixed togheter. */

  #define K_sHOME    iup_XkeyShift(K_HOME   )
  #define K_sUP      iup_XkeyShift(K_UP     )
  #define K_sPGUP    iup_XkeyShift(K_PGUP   )
  #define K_sLEFT    iup_XkeyShift(K_LEFT   )
  #define K_sMIDDLE  iup_XkeyShift(K_MIDDLE )
  #define K_sRIGHT   iup_XkeyShift(K_RIGHT  )
  #define K_sEND     iup_XkeyShift(K_END    )
  #define K_sDOWN    iup_XkeyShift(K_DOWN   )
  #define K_sPGDN    iup_XkeyShift(K_PGDN   )
  #define K_sINS     iup_XkeyShift(K_INS    )
  #define K_sDEL     iup_XkeyShift(K_DEL    )
  #define K_sSP      iup_XkeyShift(K_SP     )
  #define K_sTAB     iup_XkeyShift(K_TAB    )
  #define K_sCR      iup_XkeyShift(K_CR     )
  #define K_sBS      iup_XkeyShift(K_BS     )
  #define K_sPAUSE   iup_XkeyShift(K_PAUSE  )
  #define K_sESC     iup_XkeyShift(K_ESC    )
  #define K_sF1      iup_XkeyShift(K_F1     )
  #define K_sF2      iup_XkeyShift(K_F2     )
  #define K_sF3      iup_XkeyShift(K_F3     )
  #define K_sF4      iup_XkeyShift(K_F4     )
  #define K_sF5      iup_XkeyShift(K_F5     )
  #define K_sF6      iup_XkeyShift(K_F6     )
  #define K_sF7      iup_XkeyShift(K_F7     )
  #define K_sF8      iup_XkeyShift(K_F8     )
  #define K_sF9      iup_XkeyShift(K_F9     )
  #define K_sF10     iup_XkeyShift(K_F10    )
  #define K_sF11     iup_XkeyShift(K_F11    )
  #define K_sF12     iup_XkeyShift(K_F12    )
  #define K_sPrint   iup_XkeyShift(K_Print  )
  #define K_sMenu    iup_XkeyShift(K_Menu   )

  #define K_cHOME     iup_XkeyCtrl(K_HOME    )
  #define K_cUP       iup_XkeyCtrl(K_UP      )
  #define K_cPGUP     iup_XkeyCtrl(K_PGUP    )
  #define K_cLEFT     iup_XkeyCtrl(K_LEFT    )
  #define K_cMIDDLE   iup_XkeyCtrl(K_MIDDLE  )
  #define K_cRIGHT    iup_XkeyCtrl(K_RIGHT   )
  #define K_cEND      iup_XkeyCtrl(K_END     )
  #define K_cDOWN     iup_XkeyCtrl(K_DOWN    )
  #define K_cPGDN     iup_XkeyCtrl(K_PGDN    )
  #define K_cINS      iup_XkeyCtrl(K_INS     )
  #define K_cDEL      iup_XkeyCtrl(K_DEL     )
  #define K_cSP       iup_XkeyCtrl(K_SP      )
  #define K_cTAB      iup_XkeyCtrl(K_TAB     )
  #define K_cCR       iup_XkeyCtrl(K_CR      )
  #define K_cBS       iup_XkeyCtrl(K_BS      )
  #define K_cPAUSE    iup_XkeyCtrl(K_PAUSE   )
  #define K_cESC      iup_XkeyCtrl(K_ESC     )
  #define K_cCcedilla iup_XkeyCtrl(K_Ccedilla)
  #define K_cF1       iup_XkeyCtrl(K_F1      )
  #define K_cF2       iup_XkeyCtrl(K_F2      )
  #define K_cF3       iup_XkeyCtrl(K_F3      )
  #define K_cF4       iup_XkeyCtrl(K_F4      )
  #define K_cF5       iup_XkeyCtrl(K_F5      )
  #define K_cF6       iup_XkeyCtrl(K_F6      )
  #define K_cF7       iup_XkeyCtrl(K_F7      )
  #define K_cF8       iup_XkeyCtrl(K_F8      )
  #define K_cF9       iup_XkeyCtrl(K_F9      )
  #define K_cF10      iup_XkeyCtrl(K_F10     )
  #define K_cF11      iup_XkeyCtrl(K_F11     )
  #define K_cF12      iup_XkeyCtrl(K_F12     )
  #define K_cPrint    iup_XkeyCtrl(K_Print   )
  #define K_cMenu     iup_XkeyCtrl(K_Menu    )

  #define K_mHOME     iup_XkeyAlt(K_HOME    )
  #define K_mUP       iup_XkeyAlt(K_UP      )
  #define K_mPGUP     iup_XkeyAlt(K_PGUP    )
  #define K_mLEFT     iup_XkeyAlt(K_LEFT    )
  #define K_mMIDDLE   iup_XkeyAlt(K_MIDDLE  )
  #define K_mRIGHT    iup_XkeyAlt(K_RIGHT   )
  #define K_mEND      iup_XkeyAlt(K_END     )
  #define K_mDOWN     iup_XkeyAlt(K_DOWN    )
  #define K_mPGDN     iup_XkeyAlt(K_PGDN    )
  #define K_mINS      iup_XkeyAlt(K_INS     )
  #define K_mDEL      iup_XkeyAlt(K_DEL     )
  #define K_mSP       iup_XkeyAlt(K_SP      )
  #define K_mTAB      iup_XkeyAlt(K_TAB     )
  #define K_mCR       iup_XkeyAlt(K_CR      )
  #define K_mBS       iup_XkeyAlt(K_BS      )
  #define K_mPAUSE    iup_XkeyAlt(K_PAUSE   )
  #define K_mESC      iup_XkeyAlt(K_ESC     )
  #define K_mCcedilla iup_XkeyAlt(K_Ccedilla)
  #define K_mF1       iup_XkeyAlt(K_F1      )
  #define K_mF2       iup_XkeyAlt(K_F2      )
  #define K_mF3       iup_XkeyAlt(K_F3      )
  #define K_mF4       iup_XkeyAlt(K_F4      )
  #define K_mF5       iup_XkeyAlt(K_F5      )
  #define K_mF6       iup_XkeyAlt(K_F6      )
  #define K_mF7       iup_XkeyAlt(K_F7      )
  #define K_mF8       iup_XkeyAlt(K_F8      )
  #define K_mF9       iup_XkeyAlt(K_F9      )
  #define K_mF10      iup_XkeyAlt(K_F10     )
  #define K_mF11      iup_XkeyAlt(K_F11     )
  #define K_mF12      iup_XkeyAlt(K_F12     )
  #define K_mPrint    iup_XkeyAlt(K_Print   )
  #define K_mMenu     iup_XkeyAlt(K_Menu    )

  #define K_yHOME     iup_XkeySys(K_HOME    )
  #define K_yUP       iup_XkeySys(K_UP      )
  #define K_yPGUP     iup_XkeySys(K_PGUP    )
  #define K_yLEFT     iup_XkeySys(K_LEFT    )
  #define K_yMIDDLE   iup_XkeySys(K_MIDDLE  )
  #define K_yRIGHT    iup_XkeySys(K_RIGHT   )
  #define K_yEND      iup_XkeySys(K_END     )
  #define K_yDOWN     iup_XkeySys(K_DOWN    )
  #define K_yPGDN     iup_XkeySys(K_PGDN    )
  #define K_yINS      iup_XkeySys(K_INS     )
  #define K_yDEL      iup_XkeySys(K_DEL     )
  #define K_ySP       iup_XkeySys(K_SP      )
  #define K_yTAB      iup_XkeySys(K_TAB     )
  #define K_yCR       iup_XkeySys(K_CR      )
  #define K_yBS       iup_XkeySys(K_BS      )
  #define K_yPAUSE    iup_XkeySys(K_PAUSE   )
  #define K_yESC      iup_XkeySys(K_ESC     )
  #define K_yCcedilla iup_XkeySys(K_Ccedilla)
  #define K_yF1       iup_XkeySys(K_F1      )
  #define K_yF2       iup_XkeySys(K_F2      )
  #define K_yF3       iup_XkeySys(K_F3      )
  #define K_yF4       iup_XkeySys(K_F4      )
  #define K_yF5       iup_XkeySys(K_F5      )
  #define K_yF6       iup_XkeySys(K_F6      )
  #define K_yF7       iup_XkeySys(K_F7      )
  #define K_yF8       iup_XkeySys(K_F8      )
  #define K_yF9       iup_XkeySys(K_F9      )
  #define K_yF10      iup_XkeySys(K_F10     )
  #define K_yF11      iup_XkeySys(K_F11     )
  #define K_yF12      iup_XkeySys(K_F12     )
  #define K_yPrint    iup_XkeySys(K_Print   )
  #define K_yMenu     iup_XkeySys(K_Menu    )

  #define K_sPlus         iup_XkeyShift(K_plus    )   
  #define K_sComma        iup_XkeyShift(K_comma   )   
  #define K_sMinus        iup_XkeyShift(K_minus   )   
  #define K_sPeriod       iup_XkeyShift(K_period  )   
  #define K_sSlash        iup_XkeyShift(K_slash   )   
  #define K_sAsterisk     iup_XkeyShift(K_asterisk)

  #define K_cA     iup_XkeyCtrl(K_A)
  #define K_cB     iup_XkeyCtrl(K_B)
  #define K_cC     iup_XkeyCtrl(K_C)
  #define K_cD     iup_XkeyCtrl(K_D)
  #define K_cE     iup_XkeyCtrl(K_E)
  #define K_cF     iup_XkeyCtrl(K_F)
  #define K_cG     iup_XkeyCtrl(K_G)
  #define K_cH     iup_XkeyCtrl(K_H)
  #define K_cI     iup_XkeyCtrl(K_I)
  #define K_cJ     iup_XkeyCtrl(K_J)
  #define K_cK     iup_XkeyCtrl(K_K)
  #define K_cL     iup_XkeyCtrl(K_L)
  #define K_cM     iup_XkeyCtrl(K_M)
  #define K_cN     iup_XkeyCtrl(K_N)
  #define K_cO     iup_XkeyCtrl(K_O)
  #define K_cP     iup_XkeyCtrl(K_P)
  #define K_cQ     iup_XkeyCtrl(K_Q)
  #define K_cR     iup_XkeyCtrl(K_R)
  #define K_cS     iup_XkeyCtrl(K_S)
  #define K_cT     iup_XkeyCtrl(K_T)
  #define K_cU     iup_XkeyCtrl(K_U)
  #define K_cV     iup_XkeyCtrl(K_V)
  #define K_cW     iup_XkeyCtrl(K_W)
  #define K_cX     iup_XkeyCtrl(K_X)
  #define K_cY     iup_XkeyCtrl(K_Y)
  #define K_cZ     iup_XkeyCtrl(K_Z)
  #define K_c1     iup_XkeyCtrl(K_1)
  #define K_c2     iup_XkeyCtrl(K_2)
  #define K_c3     iup_XkeyCtrl(K_3)
  #define K_c4     iup_XkeyCtrl(K_4)
  #define K_c5     iup_XkeyCtrl(K_5)
  #define K_c6     iup_XkeyCtrl(K_6)
  #define K_c7     iup_XkeyCtrl(K_7)        
  #define K_c8     iup_XkeyCtrl(K_8)         
  #define K_c9     iup_XkeyCtrl(K_9)
  #define K_c0     iup_XkeyCtrl(K_0)
  #define K_cPlus         iup_XkeyCtrl(K_plus        )   
  #define K_cComma        iup_XkeyCtrl(K_comma       )   
  #define K_cMinus        iup_XkeyCtrl(K_minus       )   
  #define K_cPeriod       iup_XkeyCtrl(K_period      )   
  #define K_cSlash        iup_XkeyCtrl(K_slash       )   
  #define K_cSemicolon    iup_XkeyCtrl(K_semicolon   ) 
  #define K_cEqual        iup_XkeyCtrl(K_equal       )
  #define K_cBracketleft  iup_XkeyCtrl(K_bracketleft )
  #define K_cBracketright iup_XkeyCtrl(K_bracketright)
  #define K_cBackslash    iup_XkeyCtrl(K_backslash   )
  #define K_cAsterisk     iup_XkeyCtrl(K_asterisk    )

  #define K_mA     iup_XkeyAlt(K_A)
  #define K_mB     iup_XkeyAlt(K_B)
  #define K_mC     iup_XkeyAlt(K_C)
  #define K_mD     iup_XkeyAlt(K_D)
  #define K_mE     iup_XkeyAlt(K_E)
  #define K_mF     iup_XkeyAlt(K_F)
  #define K_mG     iup_XkeyAlt(K_G)
  #define K_mH     iup_XkeyAlt(K_H)
  #define K_mI     iup_XkeyAlt(K_I)
  #define K_mJ     iup_XkeyAlt(K_J)
  #define K_mK     iup_XkeyAlt(K_K)
  #define K_mL     iup_XkeyAlt(K_L)
  #define K_mM     iup_XkeyAlt(K_M)
  #define K_mN     iup_XkeyAlt(K_N)
  #define K_mO     iup_XkeyAlt(K_O)
  #define K_mP     iup_XkeyAlt(K_P)
  #define K_mQ     iup_XkeyAlt(K_Q)
  #define K_mR     iup_XkeyAlt(K_R)
  #define K_mS     iup_XkeyAlt(K_S)
  #define K_mT     iup_XkeyAlt(K_T)
  #define K_mU     iup_XkeyAlt(K_U)
  #define K_mV     iup_XkeyAlt(K_V)
  #define K_mW     iup_XkeyAlt(K_W)
  #define K_mX     iup_XkeyAlt(K_X)
  #define K_mY     iup_XkeyAlt(K_Y)
  #define K_mZ     iup_XkeyAlt(K_Z)
  #define K_m1     iup_XkeyAlt(K_1)
  #define K_m2     iup_XkeyAlt(K_2)
  #define K_m3     iup_XkeyAlt(K_3)
  #define K_m4     iup_XkeyAlt(K_4)
  #define K_m5     iup_XkeyAlt(K_5)
  #define K_m6     iup_XkeyAlt(K_6)
  #define K_m7     iup_XkeyAlt(K_7)        
  #define K_m8     iup_XkeyAlt(K_8)         
  #define K_m9     iup_XkeyAlt(K_9)
  #define K_m0     iup_XkeyAlt(K_0)
  #define K_mPlus         iup_XkeyAlt(K_plus        )   
  #define K_mComma        iup_XkeyAlt(K_comma       )   
  #define K_mMinus        iup_XkeyAlt(K_minus       )   
  #define K_mPeriod       iup_XkeyAlt(K_period      )   
  #define K_mSlash        iup_XkeyAlt(K_slash       )   
  #define K_mSemicolon    iup_XkeyAlt(K_semicolon   ) 
  #define K_mEqual        iup_XkeyAlt(K_equal       )
  #define K_mBracketleft  iup_XkeyAlt(K_bracketleft )
  #define K_mBracketright iup_XkeyAlt(K_bracketright)
  #define K_mBackslash    iup_XkeyAlt(K_backslash   )
  #define K_mAsterisk     iup_XkeyAlt(K_asterisk    )

  #define K_yA     iup_XkeySys(K_A)
  #define K_yB     iup_XkeySys(K_B)
  #define K_yC     iup_XkeySys(K_C)
  #define K_yD     iup_XkeySys(K_D)
  #define K_yE     iup_XkeySys(K_E)
  #define K_yF     iup_XkeySys(K_F)
  #define K_yG     iup_XkeySys(K_G)
  #define K_yH     iup_XkeySys(K_H)
  #define K_yI     iup_XkeySys(K_I)
  #define K_yJ     iup_XkeySys(K_J)
  #define K_yK     iup_XkeySys(K_K)
  #define K_yL     iup_XkeySys(K_L)
  #define K_yM     iup_XkeySys(K_M)
  #define K_yN     iup_XkeySys(K_N)
  #define K_yO     iup_XkeySys(K_O)
  #define K_yP     iup_XkeySys(K_P)
  #define K_yQ     iup_XkeySys(K_Q)
  #define K_yR     iup_XkeySys(K_R)
  #define K_yS     iup_XkeySys(K_S)
  #define K_yT     iup_XkeySys(K_T)
  #define K_yU     iup_XkeySys(K_U)
  #define K_yV     iup_XkeySys(K_V)
  #define K_yW     iup_XkeySys(K_W)
  #define K_yX     iup_XkeySys(K_X)
  #define K_yY     iup_XkeySys(K_Y)
  #define K_yZ     iup_XkeySys(K_Z)
  #define K_y1     iup_XkeySys(K_1)
  #define K_y2     iup_XkeySys(K_2)
  #define K_y3     iup_XkeySys(K_3)
  #define K_y4     iup_XkeySys(K_4)
  #define K_y5     iup_XkeySys(K_5)
  #define K_y6     iup_XkeySys(K_6)
  #define K_y7     iup_XkeySys(K_7)        
  #define K_y8     iup_XkeySys(K_8)         
  #define K_y9     iup_XkeySys(K_9)
  #define K_y0     iup_XkeySys(K_0)
  #define K_yPlus         iup_XkeySys(K_plus        )   
  #define K_yComma        iup_XkeySys(K_comma       )   
  #define K_yMinus        iup_XkeySys(K_minus       )   
  #define K_yPeriod       iup_XkeySys(K_period      )   
  #define K_ySlash        iup_XkeySys(K_slash       )   
  #define K_ySemicolon    iup_XkeySys(K_semicolon   ) 
  #define K_yEqual        iup_XkeySys(K_equal       )
  #define K_yBracketleft  iup_XkeySys(K_bracketleft )
  #define K_yBracketright iup_XkeySys(K_bracketright)
  #define K_yBackslash    iup_XkeySys(K_backslash   )
  #define K_yAsterisk     iup_XkeySys(K_asterisk    )


  #endif

  # -- from CD library

  CD_QUERY = -1             # query value 

  # bitmap type
  # these definitions are compatibility with the IM library
  CD_RGB = 0
  CD_MAP = 1
  CD_RGBA = 0x100 

  # bitmap data
  CD_IRED = 0
  CD_IGREEN = 1
  CD_IBLUE = 2
  CD_IALPHA = 3
  CD_INDEX = 4
  CD_COLORS = 5

  # status report
  CD_ERROR = -1
  CD_OK    =  0

  # clip mode 
  CD_CLIPOFF = 0
  CD_CLIPAREA = 1
  CD_CLIPPOLYGON = 2
  CD_CLIPREGION = 3

  # region combine mode
  CD_UNION = 0
  CD_INTERSECT = 1
  CD_DIFFERENCE = 2
  CD_NOTINTERSECT = 3

  # :nodoc:
  # polygon mode (begin...end)
  CD_FILL = 0
  CD_OPEN_LINES = 1
  CD_CLOSED_LINES = 2
  CD_CLIP = 3
  CD_BEZIER = 4
  CD_REGION = 5
  CD_PATH = 6

  CD_POLYCUSTOM = 10

  # path actions
  CD_PATH_NEW = 0
  CD_PATH_MOVETO = 1
  CD_PATH_LINETO = 2
  CD_PATH_ARC = 3
  CD_PATH_CURVETO = 4
  CD_PATH_CLOSE = 5
  CD_PATH_FILL = 6
  CD_PATH_STROKE = 7
  CD_PATH_FILLSTROKE = 7
  CD_PATH_CLIP = 8

  # fill mode
  CD_EVENODD = 0
  CD_WINDING = 1

  # line join
  CD_MITER = 0
  CD_BEVEL = 1
  CD_ROUND = 2

  # line cap 
  CD_CAPFLAT = 0
  CD_CAPSQUARE = 1
  CD_CAPROUND = 2

  # background opacity mode 
  CD_OPAQUE = 0
  CD_TRANSPARENT = 1

  # write mode 
  CD_REPLACE = 0
  CD_XOR = 1
  CD_NOT_XOR = 2

  # :doc:
  # color allocation mode (palette)
  CD_POLITE = 0
  CD_FORCE = 1

  # :nodoc:
  # line style 
  CD_CONTINUOUS = 0
  CD_DASHED = 1
  CD_DOTTED = 2
  CD_DASH_DOT = 3
  CD_DASH_DOT_DOT = 4
  CD_CUSTOM = 5

  # marker type 
  CD_PLUS = 0
  CD_STAR = 1
  CD_CIRCLE = 2
  CD_X = 3
  CD_BOX = 4
  CD_DIAMOND = 5
  CD_HOLLOW_CIRCLE = 6
  CD_HOLLOW_BOX = 7
  CD_HOLLOW_DIAMOND = 8

  # hatch type
  CD_HORIZONTAL = 0
  CD_VERTICAL = 1
  CD_FDIAGONAL = 2
  CD_BDIAGONAL = 3
  CD_CROSS = 4
  CD_DIAGCROSS = 5 

  # interior style
  CD_SOLID = 0
  CD_HATCH = 1
  CD_STIPPLE = 2
  CD_PATTERN = 3
  CD_HOLLOW = 4

  # text alignment 
  CD_NORTH = 0
  CD_SOUTH = 1
  CD_EAST = 2
  CD_WEST = 3 
  CD_NORTH_EAST = 4
  CD_NORTH_WEST = 5
  CD_SOUTH_EAST = 6
  CD_SOUTH_WEST = 7
  CD_CENTER = 8
  CD_BASE_LEFT = 9
  CD_BASE_CENTER = 10
  CD_BASE_RIGHT = 11

  # style 
  CD_PLAIN  = 0
  CD_BOLD   = 1
  CD_ITALIC = 2
  CD_UNDERLINE = 4
  CD_STRIKEOUT = 8

  CD_BOLD_ITALIC  = (CD_BOLD|CD_ITALIC)  # compatibility name
  # :doc:

  # some font sizes
  CD_SMALL    =  8
  CD_STANDARD = 12
  CD_LARGE    = 18

  # context capabilities
  CD_CAP_NONE = 0x00000000
  CD_CAP_FLUSH = 0x00000001
  CD_CAP_CLEAR = 0x00000002
  CD_CAP_PLAY = 0x00000004
  CD_CAP_YAXIS = 0x00000008
  CD_CAP_CLIPAREA = 0x00000010
  CD_CAP_CLIPPOLY = 0x00000020
  CD_CAP_REGION = 0x00000040
  CD_CAP_RECT = 0x00000080
  CD_CAP_CHORD = 0x00000100
  CD_CAP_IMAGERGB = 0x00000200
  CD_CAP_IMAGERGBA = 0x00000400
  CD_CAP_IMAGEMAP = 0x00000800
  CD_CAP_GETIMAGERGB = 0x00001000
  CD_CAP_IMAGESRV = 0x00002000
  CD_CAP_BACKGROUND = 0x00004000
  CD_CAP_BACKOPACITY = 0x00008000
  CD_CAP_WRITEMODE = 0x00010000
  CD_CAP_LINESTYLE = 0x00020000
  CD_CAP_LINEWITH = 0x00040000
  CD_CAP_FPRIMTIVES = 0x00080000
  CD_CAP_HATCH = 0x00100000
  CD_CAP_STIPPLE = 0x00200000
  CD_CAP_PATTERN = 0x00400000
  CD_CAP_FONT = 0x00800000
  CD_CAP_FONTDIM = 0x01000000
  CD_CAP_TEXTSIZE = 0x02000000
  CD_CAP_TEXTORIENTATION = 0x04000000
  CD_CAP_PALETTE = 0x08000000
  CD_CAP_LINECAP = 0x10000000
  CD_CAP_LINEJOIN = 0x20000000
  CD_CAP_PATH = 0x40000000
  CD_CAP_BEZIER = 0x80000000
  CD_CAP_ALL = 0xFFFFFFFF

  # simulation flags 
  CD_SIM_NONE = 0x0000
  CD_SIM_LINE = 0x0001
  CD_SIM_RECT = 0x0002
  CD_SIM_BOX = 0x0004
  CD_SIM_ARC = 0x0008
  CD_SIM_SECTOR = 0x0010
  CD_SIM_CHORD = 0x0020
  CD_SIM_POLYLINE = 0x0040
  CD_SIM_POLYGON = 0x0080
  CD_SIM_TEXT = 0x0100
  CD_SIM_ALL = 0xFFFF

  CD_SIM_LINES = (CD_SIM_LINE | CD_SIM_RECT | CD_SIM_ARC | CD_SIM_POLYLINE)
  CD_SIM_FILLS = (CD_SIM_BOX | CD_SIM_SECTOR | CD_SIM_CHORD | CD_SIM_POLYGON)

  # :nodoc:
  # some predefined colors for convenience
  CD_RED =           0xFF0000   # 255,  0,  0
  CD_DARK_RED =      0x800000   # 128,  0,  0
  CD_GREEN =         0x00FF00   #   0,255,  0
  CD_DARK_GREEN =    0x008000   #   0,128,  0
  CD_BLUE =          0x0000FF   #   0,  0,255
  CD_DARK_BLUE =     0x000080   #   0,  0,128

  CD_YELLOW =        0xFFFF00   # 255,255,  0
  CD_DARK_YELLOW =   0x808000   # 128,128,  0
  CD_MAGENTA =       0xFF00FF   # 255,  0,255
  CD_DARK_MAGENTA =  0x800080   # 128,  0,128
  CD_CYAN =          0x00FFFF   #   0,255,255
  CD_DARK_CYAN =     0x008080   #   0,128,128

  CD_WHITE =         0xFFFFFF   # 255,255,255
  CD_BLACK =         0x000000   #   0,  0,  0

  CD_DARK_GRAY =     0x808080   # 128,128,128
  CD_GRAY =          0xC0C0C0   # 192,192,192

  # :doc:
  # some useful conversion factors 
  CD_MM2PT   =  2.834645669   # milimeters to points (pt = CD_MM2PT * mm) 
  CD_RAD2DEG = 57.295779513   # radians to degrees (deg = CD_RAD2DEG * rad) 
  CD_DEG2RAD = 0.01745329252  # degrees to radians (rad = CD_DEG2RAD * deg) 

  # SHOW_CB Callback Values
  IUP_SHOW = 0
  IUP_RESTORE = 1
  IUP_MINIMIZE = 2
  IUP_MAXIMIZE = 3
  IUP_HIDE = 4

  # :nodoc:

  # SCROLL_CB Callback Values
  # -- actions on vertical scrollbar
  IUP_SBUP = 0
  IUP_SBDN = 1
  IUP_SBPGUP = 2
  IUP_SBPGDN = 3
  IUP_SBPOSV = 4
  IUP_SBDRAGV = 5
  # -- actions on horizontal scrollbar
  IUP_SBLEFT = 6 
  IUP_SBRIGHT = 7
  IUP_SBPGLEFT = 8
  IUP_SBPGRIGHT = 9
  IUP_SBPOSH = 10
  IUP_SBDRAGH = 11

  # :doc:

  # used by ColourBar
  IUP_PRIMARY = -1
  IUP_SECONDARY = -2
end

