module Iup

  # defines attributes and methods for drag and drop
  #
  # Steps to use the Drag & Drop support in an IUP application:
  #
  # AT SOURCE::
  #        * Enable the element as source using the attribute DRAGSOURCE=YES;
  #        * Define the data types supported by the element for the drag operation using the DRAGTYPES attribute;
  #        * Register the required callbacks DRAGBEGIN_CB, DRAGDATASIZE_CB and DRAGDATA_CB for drag handling. DRAGEND_CB is the only optional drag callback, all other callbacks and attributes must be set.
  # AT TARGET::
  #        * Enable the element as target using the attribute DROPTARGET=YES;
  #        * Define the data types supported by the element for the drop using the DROPTYPES attribute;
  #        * Register the required callback DROPDATA_CB for handling the data received. This callback and all the drop target attributes must be set too. DROPMOTION_CB is the only optional drop callback.
  #
  module DragDropAttributes
    extend AttributeBuilders

    # attributes for drag-n-drop

    define_attribute :dragsource
    define_attribute :dragtypes
    define_attribute :dragsourcemove
    define_attribute :droptarget
    define_attribute :droptypes

    # callbacks for drag-n-drop

    def dragbegin_cb callback
      define_callback callback, 'DRAGBEGIN_CB', :ii_i
    end 

    def dragdatasize_cb callback
      define_callback callback, 'DRAGDATASIZE_CB', :s_i
    end 

    # Note: user data assumed to be a string
    def dragdata_cb callback
      define_callback callback, 'DRAGDATA_CB', :ssi_i
    end 

    def dragend_cb callback
      define_callback callback, 'DRAGEND_CB', :i_i
    end 

    # Note: user data assumed to be a string
    def dropdata_cb callback
      define_callback callback, 'DROPDATA_CB', :ssiii_i
    end

    def dropmotion_cb callback
      define_callback callback, 'DROPMOTION_CB', :iis_i
    end
  end

end

