module Iup

  # Methods for containers where the child elements can be added after construction.
  #
  module DynamicFillMethods

    # Inserts an interface element at the end of the container, 
    # after the last element of the container. 
    # Valid for any element that contains other elements like 
    # dialog, frame, hbox, vbox, zbox or menu.
    def append child
      IupLib.IupAppend @handle, child.handle
    end

    # Inserts an interface element before another child of the container. 
    # Valid for any element that contains other elements like 
    # dialog, frame, hbox, vbox, zbox, menu, etc.
    def insert ref_child, new_child
      if ref_child.nil?
        IupLib.IupInsert @handle, nil, new_child.handle
      else
        IupLib.IupInsert @handle, ref_child.handle, new_child.handle
      end
    end
  end

end
