module Iup

  # An expandable container for a single widget. 
  # The user can show or hide the child widget, as required.
  #
  # == Attributes
  #
  # autoshow:: If set, child will be shown when mouse hovers over container.
  #            Values 'yes' / 'no'.
  # backcolor:: Colour of background in title area, as "r g b".
  # barposition:: Position of expander bar, as 'top' / 'bottom' / 'left' / 'right'.
  # barsize:: n, size of bar - defaults to text line height + 5.
  # clientoffset:: <b>read-only</b>, returns current offset of box in its client
  #                as "widthxheight".
  # clientsize:: <b>read-only</b>, returns current size of box as "widthxheight".
  # expand:: Allows container to fill available space in indicated direction.
  #          Values 'no' / 'horizontal' / 'vertical' / 'yes'.
  # forecolor:: Colour of text in title area, as "r g b".
  # position:: <b>read-only</b> returns position in pixels within client window
  #            as "x,y".
  # rastersize:: Size of the container, in pixels, value as "widthxheight".
  # state:: Show or hide child, as 'open' / 'close'.
  # title:: Displayed on box in title area.
  #
  class Expander < Widget

    # Creates an instance of the Expander.
    # child:: the child widget
    # block:: optional block to set up the container's attributes.
    def initialize child, &block
      @handle = IupLib.IupExpander child.handle

      self.instance_eval &block if block_given?
    end

    # -- attributes

    define_attribute :autoshow
    define_attribute :backcolor
    define_attribute :barposition
    define_attribute :barsize
    define_readonly :clientoffset
    define_readonly :clientsize
    define_attribute :expand
    define_attribute :forecolor
    define_attribute :position
    define_attribute :rastersize
    define_attribute :state
    define_attribute :title

    # -- callbacks

    # Action generated after the expander state is interactively changed.
    # callback:: a zero-argument procedure
    def action_cb callback
      unless callback.arity.zero?
        raise ArgumentError, 'action_cb callback must take 0 arguments'
      end
      cb = Proc.new do |ih|
        callback.call
      end
      define_callback cb, 'ACTION_CB', :plain
    end
  end
end
