module Iup

  # A FontDialog allows the user to select a font.
  #
  # == Attributes
  #
  # parentdialog:: This dialog will be always in front of the parent dialog. 
  #                If the parent is minimized, this dialog is automatically minimized.
  #                *Important* Closing the parent will also close the child, but the 
  #                child dialog's CLOSE_CB method will not be called.
  # previewtext:: The text to use for preview of font selection.
  # status:: <b>read-only</b> returns '1' if 'OK' pressed, or null
  # title:: Title text for the message dialog.
  # value:: Initial value of font for dialog, and return value if 'OK' pressed.
  #
  class FontDialog < Widget
    
    # Creates a dialog, using the optional block to set its attributes.
    def initialize &block
      @handle = IupLib.IupFontDlg

      self.instance_eval &block if block_given?
    end

    # Shows the dialog at position x, y.
    def popup x=0, y=0
      IupLib.IupPopup @handle, x, y
    end

    # -- attributes

    def parentdialog parent=nil #:nodoc:
      attribute_reference 'PARENTDIALOG', Dialog, parent
    end

    define_attribute :previewtext
    define_readonly :status
    define_attribute :title
    define_attribute :value
  end
end

