module Iup

  # A frame contains a child widget and displays it with a border.
  # Optionally, the frame can have a text title.
  #
  # == Attributes
  #
  # clientoffset:: read-only, returns current offset of frame in its client
  #                as "widthxheight".
  # clientsize:: read-only, returns current size of frame as "widthxheight".
  # expand:: Allows frame to fill available space in indicated direction.
  #          Values 'no' / 'horizontal' / 'vertical' / 'yes'.
  # position:: <b>read-only</b> returns position in pixels within client window
  #            as "x,y".
  # rastersize:: Size of the frame, in pixels, value as "widthxheight".
  # screenposition:: <b>read-only</b> returns position in pixels on screen
  #                  as "x,y".
  # sunken:: For frame with no title, gives a sunken appearance if set: 
  #          values as 'yes' / 'no'.
  # title:: Text displayed as frame title.
  #
  class Frame < Widget

    # Creates an instance of the frame.
    # widget:: the child widget to contain
    # block:: optional block to set up the box's attributes.
    def initialize widget, &block
      @handle = IupLib.IupFrame widget.handle

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end

    # -- attributes

    define_readonly :clientoffset
    define_readonly :clientsize
    define_attribute :expand
    define_readonly :position
    define_attribute :rastersize
    define_readonly :screenposition
    define_attribute :sunken
    define_attribute :title
    
  end
end

