module Iup

  # A container which arranges its widgets in a left-to-right, top-to-bottom order.
  #
  # == Attributes
  # 
  # alignmentcol:: gets/sets horizontal alignment of columns.
  # * alignmentcol() -> reads value of 'ALIGNMENTCOL'
  # * alignmentcol(val) -> sets 'ALIGNMENTCOL' to +val+, which is 'ALEFT' / 'ACENTER' / 'ARIGHT'
  # * alignmentcol(n, val) -> sets alignment of column +n+ to +val+.  
  # alignmentlin:: vertical alignment within line, as 'atop' / 'abottom' / 'acenter'
  # cgapcol:: n, horizontal space in characters between columns.
  # cgaplin:: n, vertical space in characters between lines.
  # clientoffset:: <b>read-only</b>, returns current offset of box in its client
  #                as "widthxheight".
  # clientsize:: <b>read-only</b>, returns current size of box as "widthxheight".
  # cmargin:: Margin in x and y directions in characters, value as "mxn".
  # expand:: Allows container to fill available space in indicated direction.
  #          Values 'no' / 'horizontal' / 'vertical' / 'yes'.
  # expandchildren:: Set to allow children to expand fully, 
  #                  values as 'yes' / 'no' / 'horizontal' / 'vertical'.
  # fittochildren:: 'column' / 'line', n -> force column/line n to fit largest element in that column/line.
  # gapcol:: n, horizontal space in pixels between columns.
  # gaplin:: n, vertical space in pixels between lines.
  # homogeneouscol:: Forces all columns to have same horizontal space, 
  #                  values as 'yes' / 'no'.
  # homogeneouslin:: Forces all lines to have same vertical space.
  #                  values as 'yes' / 'no'.
  # margin:: Margin in x and y directions in pixels, value as "mxn".
  # normalsize:: Set to make natural size of children same, 
  #              values as 'yes' / 'no' / 'horizontal' / 'vertical'.
  # numcol:: <b>read-only</b> returns number of columns.
  # numdiv:: 'auto' / n, controls number of divisions in direction as 
  #          determined by +orientation+.
  # numlin:: <b>read-only</b> returns number of lines.
  # orientation:: Controls distribution of children, in lines or columns.
  #               Value as 'horizontal' / 'vertical'.
  # position:: <b>read-only</b> returns position in pixels within client window
  #            as "x,y".
  # rastersize:: Size of the container, in pixels, value as "widthxheight".
  # sizecol:: Index of column to use for calculating height of lines.
  # sizelin:: Index of line to use for calculating width of columns.
  #
  #
  class GridBox < Widget
    include DynamicFillMethods

    # Creates an instance of the hbox.
    # *widgets:: one or more child widgets 
    # block:: optional block to set up the box's attributes.
    def initialize *widgets, &block
      @handle = IupLib.IupGridBox *widget_list(widgets)

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end   

    # -- attributes

    def alignmentcol val=nil, val2=nil # :nodoc:
      if val.nil?
        IupLib.IupGetAttribute(@handle, 'ALIGNMENTCOL').first
      elsif val2.nil?
        IupLib.IupSetAttribute @handle, 'ALIGNMENTCOL', val.to_s
      else
        IupLib.IupSetAttribute @handle, "ALIGNMENTCOL#{val}", val2.to_s
      end
    end

    define_attribute :alignmentlin
    define_attribute :cgapcol
    define_attribute :cgaplin
    define_readonly :clientoffset
    define_readonly :clientsize
    define_attribute :cmargin
    define_attribute :expand
    define_attribute :expandchildren
    define_attribute :fittochildren
    define_attribute :gapcol
    define_attribute :gaplin
    define_attribute :homogeneouscol
    define_attribute :homogeneouslin
    define_attribute :margin
    define_attribute :normalizesize
    define_readonly :numcol
    define_attribute :numdiv
    define_readonly :numlin
    define_attribute :orientation
    define_readonly :position
    define_attribute :rastersize
    define_attribute :sizecol
    define_attribute :sizelin
  end
end
