module Iup

  # All IUP GUI code must be called via a block passed to this method, as 
  # in this minimal example.
  #
  #   mainloop do
  #   
  #     label = Label.new("Hello World!")
  #   
  #     Dialog.new label do
  #       title ' ... from IUP'
  #       size '150x50'
  #     end.show
  #   end
  #
  def mainloop 
    begin
      IupLib.IupOpen 0, nil
      ImgLib::IupImageLibOpen()
      yield
      IupLib.IupMainLoop
    ensure
      IupLib.IupClose
    end
  end

  # Accesses the global variable 'name'.
  # Provide a value for 'val' to set the variable, 
  # or leave as nil to get the variable.
  def global name, val=nil
    if val.nil?
      IupLib.IupGetGlobal(name).first
    else
      IupLib.IupSetGlobal name, val
    end
  end

  # Sets the idle_action
  def idle callback
    unless callback.nil? or callback.arity.zero? 
      raise ArgumentError 'callback to idle must take no arguments'
    end
    IupLib.IupSetFunction 'IDLE_ACTION', callback
  end

  # Make all of Iup module available at class level
  # so Iup::mainloop etc can be used within 'include'
  class << self 
    include Iup
  end
end
