module Iup

  # A MessageDialog is used to display a message along with one or more buttons.
  #
  # Also see: Iup#message 
  #
  # == Attributes
  #
  # buttondefault:: Index of default button, '1' / '2' / '3'.
  # buttonresponse:: Index of the pressed button, '1' / '2' / '3'.
  # buttons:: Choice of buttons to display: 'OK' / 'OKCANCEL' / 'RETRYCANCEL' / 'YESNO' / 'YESNOCANCEL'. 
  #           If help_cb is defined, then a 'help' button is also displayed.
  # dialogtype:: Selects the icon to display: 
  #              'error' / 'information' / 'message' / 'question' / 'warning'.
  # parentdialog:: This dialog will be always in front of the parent dialog. 
  #                If the parent is minimized, this dialog is automatically minimized.
  #                *Important* Closing the parent will also close the child, but the 
  #                child dialog's CLOSE_CB method will not be called.
  # title:: Title text for the message dialog.
  # value:: Message text to display within the dialog.
  #
  class MessageDialog < Widget

    # Creates a dialog, using the optional block to set its attributes.
    def initialize &block
      @handle = IupLib.IupMessageDlg

      self.instance_eval &block if block_given?
    end

    # Shows the dialog at position x, y.
    def popup x=0, y=0
      IupLib.IupPopup @handle, x, y
    end

    # -- attributes

    define_attribute :buttondefault
    define_attribute :buttonresponse
    define_attribute :buttons
    define_attribute :dialogtype

    def parentdialog parent=nil # :nodoc:
      attribute_reference 'PARENTDIALOG', Dialog, parent
    end

    define_attribute :title
    define_attribute :value
  end
end

