module Iup

  # A progress bar control, showing a percent-complete value.
  #
  # == Attributes
  #
  # dashed:: Changes style of progress bar to a dashed pattern, as 'yes' / 'no'.
  # expand:: Allows bar to fill available space in indicated direction.
  #          Values 'no' / 'horizontal' / 'vertical' / 'yes'.
  # marquee:: Displays an undefined state if set. Values 'yes' / 'no'.
  # max:: maximum value, default 1.
  # min:: minimum value, default 0.
  # orientation:: 'horizontal' / 'vertical'.
  # position:: <b>read-only</b> returns position in pixels within client window
  #            as "x,y".
  # rastersize:: Size of the widget, in pixels, value as "widthxheight".
  # screenposition:: <b>read-only</b> returns position in pixels on screen
  #                  as "x,y".
  # tip:: Tooltip string.
  # value:: number between +min+ and +max+, for current position.
  #
  class ProgressBar < Widget

    # Creates an instance of a progress bar.
    # block:: optional block to set up the widget's appearance.
    def initialize &block
      @handle = IupLib.IupProgressBar

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end

    # -- attributes

    define_attribute :dashed
    define_attribute :expand
    define_attribute :marquee
    define_attribute :max
    define_attribute :min
    define_attribute :orientation
    define_readonly :position
    define_attribute :rastersize
    define_readonly :screenposition
    define_attribute :tip
    define_attribute :value

  end
end
