module Iup

  # Scintilla widget
  #--
  # TODO: This class's actions are incomplete
  #
  class Scintilla < Widget
    @@opened = false

    def initialize &block
      unless @@opened
        # make sure the scintilla library is opened on first use
        ScintillaLib.IupScintillaOpen
        @@opened = true
      end
      @handle = ScintillaLib.IupScintilla

      self.instance_eval &block if block_given?
    end

    # -- general

    define_attribute :border
    define_attribute :canfocus
    define_attribute :clipboard
    define_attribute :cursor
    define_attribute :overwrite
    define_attribute :readonly
    define_attribute :savedstate
    define_attribute :size
    define_attribute :usepopup
    define_attribute :visiblecolumns
    define_attribute :visiblelines
    define_readonly :visiblelinescount
    define_attribute :wordwrap
    define_attribute :wordwrapvisualflags

    # -- text retrieval / modification

    define_writeonly :append
    define_id_readonly :char
    define_writeonly :clearall
    define_readonly :count
    define_writeonly :deleterange
    define_id_writeonly :insert
    define_id_readonly :line
    define_readonly :linecount
    define_readonly :linevalue
    define_writeonly :prepend
    define_attribute :value

    # -- annotation

    define_id_attribute :annotationtext
    define_id_attribute :annotationstyle
    define_attribute :annotationstyleoffset
    define_attribute :annotationvisible
    define_attribute :annotationclearall

    # -- auto-completion

    define_id_writeonly :autocshow
    define_writeonly :autoccancel
    define_readonly :autocactive
    define_readonly :autocposstart
    define_writeonly :autoccomplete
    define_writeonly :autocselect
    define_readonly :autocselectedindex
    define_attribute :autocdroprestofword
    define_attribute :autocmaxheight
    define_attribute :autocmaxwidth

    # -- brace highlighting

    define_writeonly :bracehighlight
    define_writeonly :bracebadlight
    define_id_readonly :bracematch

    # -- caret and selection

    define_attribute :caret
    define_attribute :caretpos
    define_writeonly :carettoview
    define_attribute :selectedtext
    define_attribute :selection
    define_attribute :selectionpos

    # -- folding

    define_writeonly :foldflags
    define_id_attribute :foldlevel
    define_writeonly :foldtoggle

    # -- lexer

    define_id_writeonly :keywords
    define_readonly :keywordsets
    define_attribute :lexerlanguage
    define_writeonly :loadlexerlibrary
    define_property_attribute :property
    define_attribute :propertynames     # TODO: better syntax needed

    # -- margins

    define_id_attribute :marginmaskfolders
    define_id_attribute :marginsensitive
    define_id_attribute :margintype
    define_id_attribute :marginwidth
    define_id_attribute :marginleft
    define_id_attribute :marginright
    define_id_attribute :margintext
    define_id_attribute :margintextstyle
    define_id_attribute :margintextclearall
    define_id_attribute :margincursor

    # -- markers

    define_property_writeonly :markerdefine
    define_id_attribute :markersymbol
    define_id_writeonly :markerfgcolor
    define_id_writeonly :bgcolor
    define_id_writeonly :markerbgcolorsel
    define_id_writeonly :markeralpha
    define_id_writeonly :markerrgbimage
    define_id_writeonly :markerrgbimagescale
    define_attribute :markerhighlight
    define_id_writeonly :markeradd
    define_id_writeonly :markerdelete
    define_id_writeonly :markerdeleteall
    define_id_readonly :markerget
    define_id_writeonly :markernext
    define_id_writeonly :markerprevious
    define_id_readonly :markerlinefromhandle
    define_writeonly :markerdeletehandle
    define_readonly :lastmarkeraddhandle
    define_readonly :lastmarkerfound

    # -- scrolling

    define_attribute :scrollbar
    define_writeonly :scrollto
    define_writeonly :scrolltocaret
    define_writeonly :scrolltopos
    define_attribute :scrollwidth

    # -- search and replace

    define_writeonly :searchintarget
    define_attribute :searchflags
    define_attribute :targetstart
    define_attribute :targetend
    define_writeonly :targetfromselection
    define_writeonly :replacetarget

    # -- style definition

    define_id_attribute :stylebgcolor
    define_id_attribute :stylebold
    define_id_attribute :stylecase
    define_id_attribute :stylecharset
    define_attribute :styleclearall
    define_id_attribute :styleeolfilled
    define_id_attribute :stylefgcolor
    define_id_attribute :stylefont
    define_id_attribute :stylefontsize
    define_id_attribute :stylefontsizefrac
    define_id_attribute :stylehotspot
    define_id_attribute :styleitalic
    define_id_attribute :stylereset
    define_id_attribute :styleunderline
    define_id_attribute :stylevisible
    define_id_attribute :styleweight

    # -- styling

    define_writeonly :cleardocumentstyle
    define_writeonly :startstyling
    define_id_writeonly :styling

    # -- tabs and indentation

    define_attribute :tabsize
    define_attribute :indentationguides
    define_attribute :highlightguide
    define_attribute :usetabs

    # -- undo and redo

    define_attribute :redo
    define_attribute :undo
    define_attribute :undocollect

    # -- white space

    define_attribute :extraascent
    define_attribute :extradescent
    define_attribute :whitespaceview
    define_attribute :whitespacesize
    define_writeonly :whitespacefgcolor
    define_writeonly :whitespacebgcolor

    # -- zooming

    define_writeonly :zoomin
    define_writeonly :zoomout
    define_attribute :zoom

    # -- standard

    define_attribute :expand
    define_readonly :position
    define_attribute :rastersize
    define_readonly :screenposition
    define_attribute :tip

    # -- callbacks

    # Action generated when the text is edited, but before its value is actually changed.
    # action takes a 4-argument callback: code, position, length, text
    # called when indexed item is selected
    # code is 1 for inserted text, 0 for deleted text
    # note: text will be null if code is 0 (deleted)
    def action callback
      unless callback.arity == 4
        raise ArgumentError, 'action callback must take 4 arguments: (code, position, length, text)'
      end
      cb = Proc.new do |ih, insert, pos, length, text|
        callback.call insert, pos, length, text
      end
      define_callback cb, 'ACTION', :iiis_i
    end

    include ButtonCallback

    # Action generated when the caret/cursor position is changed.
    # caret_cb takes a callback which accepts 3 arguments (line, column, position) of caret
    def caret_cb callback
      unless callback.arity == 3
        raise ArgumentError, 'caret_cb callback must take 3 arguments: (line, column, position)'
      end
      cb = Proc.new do |ih, lin, col, pos|
        callback.call lin, col, pos
      end
      define_callback cb, 'CARET_CB', :iii_i
    end

    # Action generated when the mouse is moved.
    # Callback takes 3 arguments: (x, y, state)
    # x:: x position of mouse
    # y:: y position of mouse
    # state:: status of mouse buttons and certain keyboard keys at the moment the event was generated.
    #--
    # TODO: include functions, as in button_cb
    #
    def motion_cb callback
      unless callback.arity == 3
        raise ArgumentError, 'motion_cb callback must take 3 arguments: (x, y, state)'
      end
      cb = Proc.new do |ih, x, y, state|
        callback.call x, y, state
      end
      define_callback cb, 'MOTION_CB', :iis_i
    end

    # Called after the value was interactively changed by the user. Called when 
    # the selection is changed or when the text is edited.
    def valuechanged_cb callback
      unless callback.arity.zero?
        raise ArgumentError, 'valuechanged_cb callback must take 0 arguments'
      end
      cb = Proc.new do |ih|
        callback.call
      end
      define_callback cb, 'VALUECHANGED_CB', :plain
    end
  end
end
