module Iup

  # A container for two child widgets, with a moveable divider between them.
  # Note that moving the divider cannot affect other widgets.
  #
  # == Attributes
  # 
  # autohide:: If set, hides a child if the child client area is smaller than
  #            the bar size. Values 'yes' / 'no'.
  # barsize:: Controls the size of the bar handler, defaults to 5.
  # color:: Colour of the bar grip, as "r g b".
  # expand:: Allows container to fill available space in indicated direction.
  #          Values 'no' / 'horizontal' / 'vertical' / 'yes'.
  # layoutdrag:: Updates the children's layout when bar moved automatically.
  #              Values 'yes' / 'no'.
  # orientation:: Bar handler can be either 'horizontal' or 'vertical'.
  # position:: <b>read-only</b> returns position in pixels within client window
  #            as "x,y".
  # rastersize:: Size of the container, in pixels, value as "widthxheight".
  # showgrip:: If set, shows the bar grip. Values 'yes' / 'no'.
  # value:: 0 <= n <= 1000 sets the proportion of the left/top area relative to whole.
  #
  class SplitBox < Widget

    # Creates an instance of the SplitBox.
    # child1:: the left or top child widget
    # child2:: the right or bottom child widget
    # block:: optional block to set up the box's attributes.
    def initialize child1, child2, &block
      @handle = IupLib.IupSplit child1.handle, child2.handle

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end

    define_attribute :autohide
    define_attribute :barsize
    define_attribute :color
    define_attribute :expand
    define_attribute :layoutdrag
    define_attribute :minmax
    define_attribute :orientation
    define_readonly :position
    define_attribute :rastersize
    define_attribute :showgrip
    define_attribute :value
  end
end
