module Iup

  # A stretchable container around one child widget.
  #
  # == Attributes
  #
  # clientoffset:: read-only, returns current offset of box in its client
  #                as "widthxheight".
  # clientsize:: read-only, returns current size of box as "widthxheight".
  # color:: Specify RGB values for stretch bar colour, as "r g b".
  # direction:: Direction and position of the stretch bar.
  #             Values as 'east' / 'west' / 'north' / 'south'.
  # expand:: Allows container to fill available space in indicated direction,
  #          but only on the side away from the stretch bar.
  #          Values 'no' / 'horizontal' / 'vertical' / 'yes'.
  # position:: <b>read-only</b> returns position in pixels within client window
  #            as "x,y".
  # rastersize:: Size of the container, in pixels, value as "widthxheight".
  #
  class StretchBox < Widget
    
    # Creates an instance of the container.
    # widget:: the child widget to contain
    # block:: optional block to set up the container's attributes.
    def initialize widget, &block
      @handle = IupLib.IupSbox widget.handle

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end

    # -- attributes

    define_attribute :clientoffset
    define_attribute :clientsize
    define_attribute :color
    define_attribute :direction
    define_attribute :expand
    define_attribute :position
    define_attribute :rastersize
  end
end
