module Iup

  # A menu item that, when selected, opens another menu.
  #
  # == Attributes
  #
  # image:: Image to show on menu.
  # title:: Label of menu.
  # wid:: <b>read-only</b> Native widget identifier.
  #
  class SubMenu < Widget

    # Creates instance of a SubMenu.
    #
    # title:: Label of menu
    # item:: child menu to open
    # block:: optional block to set menu's attributes
    def initialize title, item, &block
      @handle = IupLib.IupSubmenu title, item.handle

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end

    # -- attributes
    
    def image val=nil # :nodoc:
      attribute_reference 'IMAGE', ImageWidget, val
    end

    define_attribute :title
    define_readonly :wid
  end
end
