module Iup

  class Timer
    extend AttributeBuilders
    include CallbackSetter

    attr_reader :handle

    def initialize &block
      @handle = IupLib.IupTimer

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end

    # must be called when Timer is finished with
    def destroy
      IupLib.IupDestroy @handle
    end

    # -- attributes

    define_attribute :time
    define_attribute :run
    define_readonly :wid

    # -- callbacks

    # Called when the time is up.
    # Return CLOSE to end application.
    def action_cb callback
      unless callback.arity.zero?
        raise ArgumentError, 'action_cb must take 0 arguments'
      end
      cb = Proc.new do |ih|
        callback.call
      end
      define_callback cb, 'ACTION_CB', :plain
    end
  end
end

