module Iup

  # A container for one or more child widgets, arranged in a vertical column.
  #
  # == Attributes
  #
  # alignment:: horizontal alignment of children, 'atop' / 'acenter' / 'abottom'
  # clientoffset:: read-only, returns current offset of box in its client
  #                as "widthxheight".
  # clientsize:: read-only, returns current size of box as "widthxheight".
  # expand:: Allows container to fill available space in indicated direction.
  #          Values 'no' / 'horizontal' / 'vertical' / 'yes'.
  # expandchildren:: Set to allow children to expand fully, values as 'yes' / 'no'.
  # gap:: Number of pixels between children, default value of 0.
  # homogeneous:: Set to force all children to get equal size, values as 'yes' / 'no'. 
  # margin:: Margin in x and y directions, value as "mxn".
  # normalsize:: Set to make natural size of children same, values as 'yes' / 'no'.
  # position:: <b>read-only</b> returns position in pixels within client window
  #            as "x,y".
  # rastersize:: Size of the container, in pixels, value as "widthxheight".
  #
  class VBox < Widget
    include DynamicFillMethods

    # Creates an instance of the vbox.
    # *widgets:: one or more child widgets 
    # block:: optional block to set up the box's attributes.
    def initialize *widgets, &block
      @handle = IupLib.IupVbox *widget_list(widgets)

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end

    # -- attributes

    define_attribute :alignment
    define_readonly :clientoffset
    define_readonly :clientsize
    define_attribute :expand
    define_attribute :expandchildren
    define_attribute :gap
    define_attribute :homogeneous
    define_attribute :margin
    define_attribute :normalizesize
    define_readonly :position
    define_attribute :rastersize
  end


end
